@@ -18,21 +18,24 @@ use Travel::Status::DE::HAFAS::Journey;
 use Travel::Status::DE::HAFAS::Location;
 use Travel::Status::DE::HAFAS::Message;
 use Travel::Status::DE::HAFAS::Polyline qw(decode_polyline);
+use Travel::Status::DE::HAFAS::Product;
 use Travel::Status::DE::HAFAS::StopFinder;
 
-our $VERSION = '5.05';
+our $VERSION = '5.06';
 
 # {{{ Endpoint Definition
 
 my %hafas_instance = (
 	DB => {
-		stopfinder  => 'https://reiseauskunft.bahn.de/bin/ajax-getstop.exe',
-		mgate       => 'https://reiseauskunft.bahn.de/bin/mgate.exe',
-		name        => 'Deutsche Bahn',
-		productbits => [qw[ice ic_ec d regio s bus ferry u tram ondemand]],
-		salt        => 'bdI8UVj4' . '0K5fvxwf',
-		languages   => [qw[de en fr es]],
-		request     => {
+		stopfinder    => 'https://reiseauskunft.bahn.de/bin/ajax-getstop.exe',
+		mgate         => 'https://reiseauskunft.bahn.de/bin/mgate.exe',
+		name          => 'Deutsche Bahn',
+		productbits   => [qw[ice ic_ec d regio s bus ferry u tram ondemand]],
+		productgroups =>
+		  [ [qw[ice ic_ec d]], [qw[regio s]], [qw[bus ferry u tram ondemand]] ],
+		salt      => 'bdI8UVj4' . '0K5fvxwf',
+		languages => [qw[de en fr es]],
+		request   => {
 			client => {
 				id   => 'DB',
 				v    => '20100000',
@@ -116,8 +119,21 @@ my %hafas_instance = (
 		mgate       => 'https://fahrplan.oebb.at/bin/mgate.exe',
 		stopfinder  => 'https://fahrplan.oebb.at/bin/ajax-getstop.exe',
 		name        => 'Österreichische Bundesbahnen',
-		productbits =>
-		  [qw[ice ice ice regio regio s bus ferry u tram ice ondemand ice]],
+		productbits => [
+			[ ice_rj => 'long distance trains' ],
+			[ sev    => 'rail replacement service' ],
+			[ ic_ec  => 'long distance trains' ],
+			[ d_n    => 'night trains and rapid trains' ],
+			[ regio  => 'regional trains' ],
+			[ s      => 'suburban trains' ],
+			[ bus    => 'busses' ],
+			[ ferry  => 'maritime transit' ],
+			[ u      => 'underground' ],
+			[ tram   => 'trams' ],
+			[ other  => 'other transit services' ]
+		],
+		productgroups =>
+		  [ qw[ice_rj ic_ec d_n], qw[regio s sev], qw[bus ferry u tram other] ],
 		request => {
 			client => {
 				id   => 'OEBB',
@@ -509,7 +525,12 @@ sub mot_mask {
 
 	my %mot_pos;
 	for my $i ( 0 .. $#{ $hafas_instance{$service}{productbits} } ) {
-		$mot_pos{ $hafas_instance{$service}{productbits}[$i] } = $i;
+		if ( ref( $hafas_instance{$service}{productbits}[$i] ) eq 'ARRAY' ) {
+			$mot_pos{ $hafas_instance{$service}{productbits}[$i][0] } = $i;
+		}
+		else {
+			$mot_pos{ $hafas_instance{$service}{productbits}[$i] } = $i;
+		}
 	}
 
 	if ( my @mots = @{ $self->{exclusive_mots} // [] } ) {
@@ -682,6 +703,20 @@ sub add_message {
 	return $message;
 }
 
+sub parse_prodL {
+	my ($self) = @_;
+
+	my $common = $self->{raw_json}{svcResL}[0]{res}{common};
+	return [
+		map {
+			Travel::Status::DE::HAFAS::Product->new(
+				common  => $common,
+				product => $_
+			)
+		} @{ $common->{prodL} }
+	];
+}
+
 sub parse_search {
 	my ($self) = @_;
 
@@ -710,14 +745,24 @@ sub parse_journey {
 		return $self;
 	}
 
+	my $prodL = $self->parse_prodL;
+
 	my @locL = map { Travel::Status::DE::HAFAS::Location->new( loc => $_ ) }
 	  @{ $self->{raw_json}{svcResL}[0]{res}{common}{locL} // [] };
 	my $journey = $self->{raw_json}{svcResL}[0]{res}{journey};
 	my @polyline;
 
-	if ( $journey->{poly} ) {
-		@polyline = decode_polyline( $journey->{poly}{crdEncYX} );
-		for my $ref ( @{ $journey->{poly}{ppLocRefL} // [] } ) {
+	my $poly = $journey->{poly};
+
+	# ÖBB
+	if ( $journey->{polyG} and @{ $journey->{polyG}{polyXL} // [] } ) {
+		$poly = $self->{raw_json}{svcResL}[0]{res}{common}{polyL}
+		  [ $journey->{polyG}{polyXL}[0] ];
+	}
+
+	if ($poly) {
+		@polyline = decode_polyline( $poly->{crdEncYX} );
+		for my $ref ( @{ $poly->{ppLocRefL} // [] } ) {
 			my $poly = $polyline[ $ref->{ppIdx} ];
 			my $loc  = $locL[ $ref->{locX} ];
 
@@ -728,6 +773,7 @@ sub parse_journey {
 
 	$self->{result} = Travel::Status::DE::HAFAS::Journey->new(
 		common   => $self->{raw_json}{svcResL}[0]{res}{common},
+		prodL    => $prodL,
 		locL     => \@locL,
 		journey  => $journey,
 		polyline => \@polyline,
@@ -746,6 +792,8 @@ sub parse_journey_match {
 		return $self;
 	}
 
+	my $prodL = $self->parse_prodL;
+
 	my @locL = map { Travel::Status::DE::HAFAS::Location->new( loc => $_ ) }
 	  @{ $self->{raw_json}{svcResL}[0]{res}{common}{locL} // [] };
 
@@ -756,6 +804,7 @@ sub parse_journey_match {
 			@{ $self->{results} },
 			Travel::Status::DE::HAFAS::Journey->new(
 				common  => $self->{raw_json}{svcResL}[0]{res}{common},
+				prodL   => $prodL,
 				locL    => \@locL,
 				journey => $result,
 				hafas   => $self,
@@ -774,6 +823,8 @@ sub parse_board {
 		return $self;
 	}
 
+	my $prodL = $self->parse_prodL;
+
 	my @locL = map { Travel::Status::DE::HAFAS::Location->new( loc => $_ ) }
 	  @{ $self->{raw_json}{svcResL}[0]{res}{common}{locL} // [] };
 	my @jnyL = @{ $self->{raw_json}{svcResL}[0]{res}{jnyL} // [] };
@@ -783,6 +834,7 @@ sub parse_board {
 			@{ $self->{results} },
 			Travel::Status::DE::HAFAS::Journey->new(
 				common  => $self->{raw_json}{svcResL}[0]{res}{common},
+				prodL   => $prodL,
 				locL    => \@locL,
 				journey => $result,
 				hafas   => $self,
@@ -871,6 +923,11 @@ sub station {
 	my @prefcounts = sort { $b->[0] <=> $a->[0] }
 	  map { [ $_, $prefc_by_loc{$_} ] } keys %prefc_by_loc;
 
+	if ( not @prefcounts ) {
+		$self->{station_info} = {};
+		return $self->{station_info};
+	}
+
 	my $loc = $locL[ $prefcounts[0][0] ];
 
 	if ($loc) {
@@ -968,7 +1025,7 @@ monitors
 
 =head1 VERSION
 
-version 5.05
+version 5.06
 
 =head1 DESCRIPTION
 
@@ -1157,8 +1214,9 @@ If no result was found or the parser / http request failed, returns undef.
 
 =item $status->messages
 
-Returns a list of Travel::Status::DE::HAFAS::Message(3pm) objects with
-service messages. Each message belongs to at least one arrival/departure.
+Returns a list of Travel::Status::DE::HAFAS::Message(3pm) objects with service
+messages. Each message belongs to at least one arrival/departure (station,
+journey) or to at least stop alongside its route (journey).
 
 =item $status->station
 
@@ -1245,11 +1303,25 @@ The non-default services (anything other than DB) are not well tested.
 
 =head1 SEE ALSO
 
-Travel::Status::DE::HAFAS::Journey(3pm), Travel::Status::DE::HAFAS::StopFinder(3pm).
+=over
+
+=item * Travel::Status::DE::HAFAS::Journey(3pm)
+
+=item * Travel::Status::DE::HAFAS::Location(3pm)
+
+=item * Travel::Status::DE::HAFAS::Message(3pm)
+
+=item * Travel::Status::DE::HAFAS::Product(3pm)
+
+=item * Travel::Status::DE::HAFAS::StopFinder(3pm)
+
+=item * Travel::Status::DE::HAFAS::Stop(3pm)
+
+=back
 
 =head1 AUTHOR
 
-Copyright (C) 2015-2023 by Birte Kristina Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2015-2024 by Birte Kristina Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
