@@ -11,12 +11,13 @@ use DateTime::Format::Strptime;
 use List::Util qw(any);
 use Travel::Status::DE::HAFAS::Stop;
 
-our $VERSION = '5.05';
+our $VERSION = '5.06';
 
 Travel::Status::DE::HAFAS::Journey->mk_ro_accessors(
 	qw(datetime sched_datetime rt_datetime
 	  is_additional is_cancelled is_partially_cancelled
 	  station station_eva platform sched_platform rt_platform operator
+	  product
 	  id name type type_long class number line line_no load delay
 	  route_end route_start origin destination direction)
 );
@@ -26,13 +27,12 @@ Travel::Status::DE::HAFAS::Journey->mk_ro_accessors(
 sub new {
 	my ( $obj, %opt ) = @_;
 
-	my @prodL = @{ $opt{common}{prodL} // [] };
-	my @opL   = @{ $opt{common}{opL}   // [] };
 	my @icoL  = @{ $opt{common}{icoL}  // [] };
 	my @tcocL = @{ $opt{common}{tcocL} // [] };
 	my @remL  = @{ $opt{common}{remL}  // [] };
 	my @himL  = @{ $opt{common}{himL}  // [] };
 
+	my $prodL   = $opt{prodL};
 	my $locL    = $opt{locL};
 	my $hafas   = $opt{hafas};
 	my $journey = $opt{journey};
@@ -45,34 +45,7 @@ sub new {
 	my $is_cancelled        = $journey->{isCncl};
 	my $partially_cancelled = $journey->{isPartCncl};
 
-	my $product  = $prodL[ $journey->{prodX} ];
-	my $name     = $product->{addName} // $product->{name};
-	my $line_no  = $product->{prodCtx}{line};
-	my $train_no = $product->{prodCtx}{num};
-	my $cat      = $product->{prodCtx}{catOut};
-	my $catlong  = $product->{prodCtx}{catOutL};
-	if ( $name and $cat and $name eq $cat and $product->{nameS} ) {
-		$name .= ' ' . $product->{nameS};
-	}
-	if ( defined $train_no and not $train_no ) {
-		$train_no = undef;
-	}
-	if (
-		    not defined $line_no
-		and defined $product->{prodCtx}{matchId}
-		and
-		( not defined $train_no or $product->{prodCtx}{matchId} ne $train_no )
-	  )
-	{
-		$line_no = $product->{prodCtx}{matchId};
-	}
-
-	my $operator;
-	if ( defined $product->{oprX} ) {
-		if ( my $opref = $opL[ $product->{oprX} ] ) {
-			$operator = $opref->{name};
-		}
-	}
+	my $product = $prodL->[ $journey->{prodX} ];
 
 	my @messages;
 	for my $msg ( @{ $journey->{msgL} // [] } ) {
@@ -89,25 +62,33 @@ sub new {
 
 	my $datetime_ref;
 
-	if ( @{ $journey->{stopL} // [] } or $journey->{stbStop}) {
-		my ($date_ref, $parse_fmt);
-		if ($jid =~ /#/) {
+	if ( @{ $journey->{stopL} // [] } or $journey->{stbStop} ) {
+		my ( $date_ref, $parse_fmt );
+		if ( $jid =~ /#/ ) {
+
 			# ÖBB Journey ID - technically we ought to use Europe/Vienna tz
 			#  but let's not get into that...
-			$date_ref = ( split( /#/, $jid ) )[12];
+			$date_ref  = ( split( /#/, $jid ) )[12];
 			$parse_fmt = '%d%m%y';
 			if ( length($date_ref) < 5 ) {
-				warn("HAFAS, not even once -- midnight crossing may be bogus -- date_ref $date_ref");
-			} elsif ( length($date_ref) == 5 ) {
+				warn(
+"HAFAS, not even once -- midnight crossing may be bogus -- date_ref $date_ref"
+				);
+			}
+			elsif ( length($date_ref) == 5 ) {
 				$date_ref = "0${date_ref}";
 			}
-		} else {
+		}
+		else {
 			# DB Journey ID
-			$date_ref = ( split( qr{[|]}, $jid ) )[4];
+			$date_ref  = ( split( qr{[|]}, $jid ) )[4];
 			$parse_fmt = '%d%m%Y';
 			if ( length($date_ref) < 7 ) {
-				warn("HAFAS, not even once -- midnight crossing may be bogus -- date_ref $date_ref");
-			} elsif ( length($date_ref) == 7 ) {
+				warn(
+"HAFAS, not even once -- midnight crossing may be bogus -- date_ref $date_ref"
+				);
+			}
+			elsif ( length($date_ref) == 7 ) {
 				$date_ref = "0${date_ref}";
 			}
 		}
@@ -117,8 +98,6 @@ sub new {
 		)->parse_datetime($date_ref);
 	}
 
-	my $class = $product->{cls};
-
 	my @stops;
 	my $route_end;
 	for my $stop ( @{ $journey->{stopL} // [] } ) {
@@ -128,6 +107,7 @@ sub new {
 			loc          => $loc,
 			stop         => $stop,
 			common       => $opt{common},
+			prodL        => $prodL,
 			hafas        => $hafas,
 			date         => $date,
 			datetime_ref => $datetime_ref,
@@ -150,14 +130,15 @@ sub new {
 
 	my $ref = {
 		id                     => $jid,
-		name                   => $name,
-		number                 => $train_no,
-		line                   => $name,
-		line_no                => $line_no,
-		type                   => $cat,
-		type_long              => $catlong,
-		class                  => $class,
-		operator               => $operator,
+		product                => $product,
+		name                   => $product->name,
+		number                 => $product->number,
+		line                   => $product->name,
+		line_no                => $product->line_no,
+		type                   => $product->type,
+		type_long              => $product->type_long,
+		class                  => $product->class,
+		operator               => $product->operator,
 		direction              => $direction,
 		is_cancelled           => $is_cancelled,
 		is_partially_cancelled => $partially_cancelled,
@@ -186,9 +167,11 @@ sub new {
 	if ( $journey->{stbStop} ) {
 		$ref->{station}        = $locL->[ $journey->{stbStop}{locX} ]->name;
 		$ref->{station_eva}    = 0 + $locL->[ $journey->{stbStop}{locX} ]->eva;
-		$ref->{sched_platform} = $journey->{stbStop}{dPlatfS};
-		$ref->{rt_platform}    = $journey->{stbStop}{dPlatfR};
-		$ref->{platform}       = $ref->{rt_platform} // $ref->{sched_platform};
+		$ref->{sched_platform} = $journey->{stbStop}{dPlatfS}
+		  // $journey->{stbStop}{dPltfS}{txt};
+		$ref->{rt_platform} = $journey->{stbStop}{dPlatfR}
+		  // $journey->{stbStop}{dPltfR}{txt};
+		$ref->{platform} = $ref->{rt_platform} // $ref->{sched_platform};
 
 		my $time_s
 		  = $journey->{stbStop}{ $hafas->{arrivals} ? 'aTimeS' : 'dTimeS' };
@@ -412,7 +395,7 @@ journey received by Travel::Status::DE::HAFAS
 
 =head1 VERSION
 
-version 5.05
+version 5.06
 
 =head1 DESCRIPTION
 
@@ -507,6 +490,13 @@ True if the journey was cancelled, false otherwise.
 
 True if part of the journey was cancelled, false otherwise.
 
+=item $journey->product
+
+Travel::Status::DE::HAFAS::Product(3pm) instance describing the product (mode
+of transport, line number / ID, operator, ...) associated with this journey.
+Note that journeys may be associated with multiple products -- see also
+C<< $journey->route >> and C<< $stop->product >>.
+
 =item $journey->rt_platform (station only)
 
 Actual arrival/departure platform.
